unit BoardView;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, ImgList;

type
	TBoardViewBoardType = (btMain, btOther);
  TBoardViewFrame = class(TFrame)
    Panel1: TPanel;
    PageControl: TPageControl;
    BoardTabSheet: TTabSheet;
    BoardTreeView: TTreeView;
    FavTabSheet: TTabSheet;
    ImageList: TImageList;
    procedure BoardTreeViewExpanded(Sender: TObject; Node: TTreeNode);
    procedure BoardTreeViewCollapsed(Sender: TObject; Node: TTreeNode);
  private
		{ Private 錾 }
		FBoardList: TStringList;
		FOtherTreeNode: TTreeNode;
    function GetSelectedBoard: string;
    procedure SetSelectedBoard(const Value: string);
    function GetSelectedBoardName: string;
  public
		{ Public 錾 }
    constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure BoardLoadFromFile(const FileName: string; BoardType: TBoardViewBoardType);
		procedure ClearBoard(BoardType: TBoardViewBoardType);
		property SelectedBoard: string read GetSelectedBoard write SetSelectedBoard;
		property SelectedBoardName: string read GetSelectedBoardName;
	end;

implementation

uses BBSUtils;

{$R *.dfm}

{ TBoardViewFrame }

procedure TBoardViewFrame.BoardLoadFromFile(const FileName: string; BoardType: TBoardViewBoardType);
var
	List: TStringList;
	I: Integer;
	Token: array [0 .. 3] of string;
	Root, Node: TTreeNode;
begin
	Root := nil;
	List := TStringList.Create;
	
	try
		ClearBoard(BoardType);
		if BoardType = btMain then
		begin
			List.LoadFromFile(FileName);

			for I := 0 to List.Count - 1 do
			begin
				if StringSplit(List[I], #9, Token) < 2 then Continue;
				if Token[0] <> '' then
				begin
					Root := BoardTreeView.Items.AddChild(nil, 'y' + Token[0] + 'z');
					Root.ImageIndex := 1;
					Root.SelectedIndex := 1;
				end else begin
					Assert(Root <> nil);
					Node := BoardTreeView.Items.AddChild(Root, Token[3]);
					Node.Data := Pointer(FBoardList.Add(Token[1] + '/' + Token[2]) + 1);
					Node.ImageIndex := 3;
				end;
			end;
		end else { btOther }
		begin
			if not Assigned(FOtherTreeNode) then
			begin
				FOtherTreeNode := BoardTreeView.Items.AddFirst(nil, 'y̑z');
				FOtherTreeNode.ImageIndex := 1;
				FOtherTreeNode.SelectedIndex := 1;
			end;
			List.LoadFromFile(FileName);
			for I := 0 to List.Count - 1 do
			begin
				if StringSplit(List[I], #9, Token) < 3 then Continue;
				Node := BoardTreeView.Items.AddChild(FOtherTreeNode, Token[2]);
				Node.Data := Pointer(FBoardList.Add(Token[0] + '/' + Token[1]) + 1);
				Node.ImageIndex := 3;
			end;
		end;
	finally
		List.Free;
	end;
end;

procedure TBoardViewFrame.ClearBoard(BoardType: TBoardViewBoardType);
var
	List: TStringList;
	Node, Node2: TTreeNode;
	I: Integer;
begin
	List := TStringList.Create;
	try
		if BoardType = btMain then
		begin
			Node := BoardTreeView.Items.GetFirstNode;
				
			while Node <> nil do
			begin
				for I := 0 to Node.Count - 1 do
					if Node.Item[I].Data <> nil then
						List.Add(Format('%.3d', [Integer(Node.Item[I].Data) - 1]));
				Node2 := Node.GetNextSibling;
				Node.Delete;
				Node := Node2;
			end;
		end else
		begin
			if Assigned(FOtherTreeNode) then
				for I := FOtherTreeNode.Count - 1 downto 0 do
				begin
					if FOtherTreeNode.Item[I].Data <> nil then
						List.Add(Format('%.3d', [Integer(FOtherTreeNode.Item[I].Data) - 1]));
					FOtherTreeNode.Item[I].Delete;
				end;
		end;

		List.Sort;
		for I := List.Count - 1 downto 0 do
			FBoardList.Delete(StrToInt(List[I]));
	finally
		List.Free;
	end;
end;

constructor TBoardViewFrame.Create(AOwner: TComponent);
begin
  inherited;
	FBoardList := TStringList.Create;
end;

destructor TBoardViewFrame.Destroy;
begin
	FBoardList.Free;
  inherited;
end;

procedure TBoardViewFrame.BoardTreeViewExpanded(Sender: TObject;
  Node: TTreeNode);
begin
	Node.ImageIndex := 2;
	Node.SelectedIndex := 2;
end;

procedure TBoardViewFrame.BoardTreeViewCollapsed(Sender: TObject;
  Node: TTreeNode);
begin
	Node.ImageIndex := 1;
	Node.SelectedIndex := 1;
end;

function TBoardViewFrame.GetSelectedBoard: string;
begin
	Result := '';

	with BoardTreeView do
	begin
		if Selected = nil then Exit;
		if Selected.Data = nil then Exit;

		Result := FBoardList[Integer(Selected.Data) - 1];
	end;
end;

procedure TBoardViewFrame.SetSelectedBoard(const Value: string);
begin

end;

function TBoardViewFrame.GetSelectedBoardName: string;
begin
	Result := '';

	with BoardTreeView do
	begin
		if Selected = nil then Exit;
		if Selected.Data = nil then Exit;

		Result := Selected.Text;
	end;
end;

end.

