unit BrowserView;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls, ImgList, ComCtrls, ToolWin, EmbeddedWB, BBSUtils, McConfig, McTypes,
	IdAsyncHTTP, MSHTML, SyncObjs, Variants, ActiveX, StdCtrls;

type
	TBrowserTabSheet = class;
  TBrowserViewFrame = class(TFrame)
    Panel: TPanel;
    TabButtonImageList: TImageList;
    TopPanel: TPanel;
    TabToolBar: TToolBar;
    TempResMaxChange: TToolButton;
    ToolButton2: TToolButton;
    ReadMore: TToolButton;
    TabItemAbort: TToolButton;
    ToolButton7: TToolButton;
    resWrite: TToolButton;
    FavButton: TToolButton;
    ToolButton4: TToolButton;
    DoSavelog: TToolButton;
    LogDelete: TToolButton;
    TabCloseButton: TToolButton;
    PageControl1: TPageControl;
    BoardLabel: TLabel;
    TitleLabel: TLabel;
    procedure PanelResize(Sender: TObject);
    procedure TopPanelResize(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
  private
    { Private 錾 }
		FConfig: TMaCherieConfig;
		FFirstTabSheet: TBrowserTabSheet;
		procedure SetConfig(const Value: TMaCherieConfig);
    procedure SetBoardText(const Value: string);
    procedure SetTitleText(const Value: string);
	public
		{ Public 錾 }
		constructor Create(AOwner: TComponent); override;
		procedure BrowserOpen(TabType: TBrowserViewTabType; Thread: T2chThreadIniFile);
		property Config: TMaCherieConfig read FConfig write SetConfig;
		property TitleText: string write SetTitleText;
		property BoardText: string write SetBoardText;
	end;
	TBrowserLoadThread = class;
	TBrowserTabSheet = class(TTabSheet)
	private
		Browser: TEmbeddedWB;
		FConfig: TMaCherieConfig;
		FThread: T2chThreadIniFile;
		FLoadThread: TBrowserLoadThread;
		FDatList: T2chThreadDat;
		FDatCount: Integer;
		FDatIndex: Integer;
		FResCount: Integer; { X }
		FDatMem: TMemoryStream;
		FBrowserLock: TCriticalSection;
		FViewTimer: TTimer;
		FFrame: TBrowserViewFrame;
		FFirstTime: Boolean;
		FShrinkedLog: Boolean;	{ Ol܂tO }
		procedure OnDatLine(const ALineText, ATerminator: string);
		procedure ThreadCatchException(Message: string);
		procedure EndThreadLoad;
		procedure WriteString(const S: string);
		procedure OnViewTimer(Sender: TObject);
		procedure OnOccurAbornTimer(Sender: TObject);
		procedure OnTabActivate(Sender: TObject);
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure OpenThread(Thread: T2chThreadIniFile);
		property Config: TMaCherieConfig read FConfig write FConfig;
		property Thread: T2chThreadIniFile read FThread write FThread;
		property Frame: TBrowserViewFrame read FFrame write FFrame;
	end;
	TBrowserLoadThread = class(TThread)
	private
		FOwner: TBrowserTabSheet;
		FHTTP: TIdAsyncHTTP;
		FLine: string;
		FTerminator: string;
		FURL: string;
		FDisconnect: Boolean;
		FExceptionMessage: string;
    FConfig: TMaCherieConfig;
		FRangeStart: Integer;
    FFreezed: Boolean;
		procedure DatOnLine(ASender: TObject; const ALineText, ATerminator: string; Param: Pointer);
		procedure CatchException;
		procedure EndDatLoad;
		procedure OnLine;
	protected
		procedure Execute; override;
	public
		constructor Create(AOwner: TBrowserTabSheet);
		destructor Destroy; override;
		property Config: TMaCherieConfig read FConfig write FConfig;
		property URL: string read FURL write FURL;
		property Disconnect: Boolean read FDisconnect write FDisconnect;
		property RangeStart: Integer read FRangeStart write FRangeStart;
		property Freezed: Boolean read FFreezed write FFreezed;
	end;

implementation

uses McMain;

{$R *.dfm}

procedure TBrowserViewFrame.BrowserOpen(TabType: TBrowserViewTabType;
	Thread: T2chThreadIniFile);
var
	TabSheet: TBrowserTabSheet;
begin
	if TabType = ttNewTab then
	begin
		if FFirstTabSheet <> nil then
		begin
			TabSheet := FFirstTabSheet;
			FFirstTabSheet := nil;
		end else begin
			TabSheet := TBrowserTabSheet.Create(nil);
			TabSheet.Config := FConfig;
			TabSheet.PageControl := PageControl1;
			TabSheet.Frame := Self;
		end;
		if PageControl1.ActivePage <> TabSheet then
			PageControl1.ActivePage := TabSheet;
	end else
		TabSheet := TBrowserTabSheet(PageControl1.ActivePage);

	TabSheet.Config := FConfig;
	TabSheet.OpenThread(Thread);
end;

constructor TBrowserViewFrame.Create(AOwner: TComponent);
var
	TabSheet: TBrowserTabSheet;
begin
  inherited;

	TabSheet := TBrowserTabSheet.Create(PageControl1);
	TabSheet.Config := FConfig;
	TabSheet.PageControl := PageControl1;
	TabSheet.Frame := Self;
	
	FFirstTabSheet := TabSheet;
end;

procedure TBrowserViewFrame.PanelResize(Sender: TObject);
begin
	TopPanel.Height := ToolButton2.Height;
end;

procedure TBrowserViewFrame.TopPanelResize(Sender: TObject);
var
	ATop: Integer;
begin
	ATop := (TopPanel.ClientHeight - BoardLabel.Height) div 2;
	BoardLabel.Top := ATop;
	TitleLabel.Top := ATop;
end;

procedure TBrowserViewFrame.SetConfig(const Value: TMaCherieConfig);
begin
	FConfig := Value;
end;

procedure TBrowserViewFrame.SetBoardText(const Value: string);
begin
	BoardLabel.Caption := Format('[%s]', [Value]);
	with BoardLabel do
		TitleLabel.Left := Left + Width + 5;
end;

procedure TBrowserViewFrame.SetTitleText(const Value: string);
begin
	TitleLabel.Caption := Value;
end;

procedure TBrowserViewFrame.PageControl1Change(Sender: TObject);
begin
	(PageControl1.ActivePage as TBrowserTabSheet).OnTabActivate(Self);
end;

{ TBrowserTabSheet }

constructor TBrowserTabSheet.Create(AOwner: TComponent);
begin
	inherited;
{	OnEnter := OnTabActivate;}
	
	Browser := TEmbeddedWB.Create(nil);
	TControl(Browser).Parent := Self;
	Browser.Loaded;
	Browser.Align := alClient;
	Browser.Visible := True;

	FViewTimer := TTimer.Create(nil);
	FViewTimer.Enabled := False;
	FViewTimer.Interval := 500;
	FViewTimer.OnTimer := OnViewTimer;
	
	FLoadThread := TBrowserLoadThread.Create(Self);
	FDatList := T2chThreadDat.Create;
	FDatMem := TMemoryStream.Create;
	FBrowserLock := TCriticalSection.Create;
end;

procedure TBrowserTabSheet.OnDatLine(const ALineText, ATerminator: string);
var
	ALine: string;
begin
	FBrowserLock.Enter;
	try
		if FFirstTime then
		begin
			{ ŏ̍sǂݍ񂾏ꍇ }
			FFirstTime := False;

			{ 擾ꍇ }
			if FThread.DatSize > 0 then
			begin
				if (ALineText = '') and ((ATerminator = #13#10) or (ATerminator = #10)) then
					Exit	{ ŏ̍sŝ݂̏ꍇAǉɔ }
				else begin
					{ DatSizePȏōŏ̍sŝ݂łȂꍇ }
					{ Ol܂tOTrueɂ }
					FShrinkedLog := True;
					FLoadThread.Disconnect := True;
					Exit;
				end;
			end;
		end;

		FDatList.Add(ALineText);
		ALine := ALineText + ATerminator;
		FDatMem.Write(PChar(ALine)^, Length(ALine));
		Inc(FDatCount);

		FThread.NewResCount := FDatList.Count - FResCount;
		FThread.ResState := rsLoading;
		FThread.ResCount := FDatList.Count;
		FThread.DateTime := Now;
		MaCherieMainForm.RefreshThread(FThread);

		if not FViewTimer.Enabled then
			FViewTimer.Enabled := True;

		if FDatCount >= 25 then
		begin
			ALine := FDatList.ToHTML(FDatIndex, FDatCount);
			if ALine <> '' then
				WriteString(ALine);
			Inc(FDatIndex, FDatCount);
			FDatCount := 0;
		end;
		FViewTimer.Enabled := False;
	finally
		FBrowserLock.Leave;
	end;
end;

destructor TBrowserTabSheet.Destroy;
begin
	FLoadThread.Disconnect := True;
	FLoadThread.Terminate;
	if FLoadThread.Suspended then
		FLoadThread.Resume;
	FLoadThread.WaitFor;
	FLoadThread.Free;

	TControl(Browser).Parent := nil;
	Browser.Free;
	FDatList.Free;
	FDatMem.Free;
	FBrowserLock.Free;
		
  inherited;
end;

procedure TBrowserTabSheet.EndThreadLoad;
var
	HTML, LogPath, DatPath: string;
	Stream: TFileStream;
	Mode: Word;
begin
	FBrowserLock.Enter;
	try
		FViewTimer.Enabled := False;

		if FDatCount > 0 then
		begin
			HTML := FDatList.ToHTML(FDatIndex, FDatCount);
			WriteString(HTML);
			FDatCount := 0;
			FDatIndex := 0;
		end;
	finally
		FBrowserLock.Leave;
	end;
	
	FConfig.Enter;
	try
		LogPath := FConfig.GetBaseDir + 'logx\' + FThread.LogPath;
		DatPath := ChangeFileExt(LogPath, '.dat');
	finally
		FConfig.Leave;
	end;

	Stream := nil;

	try
		{ fBNg쐬 }
		ForceDirectories(ExtractFileDir(DatPath));
		FThread.DatSize := FThread.DatSize + FDatMem.Size;
		FThread.Freezed := FLoadThread.Freezed;
		if FThread.Modified then
			FThread.SaveToFile(LogPath);
		
		{ DATTCYXV }
		if FDatMem.Size > 0 then
		begin
			if FileExists(DatPath)
				then Mode := fmOpenReadWrite
				else Mode := fmCreate;
			Stream := TFileStream.Create(DatPath, Mode or fmShareDenyWrite);

			Stream.Seek(0, soFromEnd);
			FDatMem.Seek(0, soFromBeginning);
			Stream.CopyFrom(FDatMem, FDatMem.Size);
		end;
		
		FDatMem.Clear;
	finally
		Stream.Free;
	end;

	if FLoadThread.Freezed then
		FThread.ResState := rsFreezed
	else if FShrinkedLog then
		FThread.ResState := rsError	{ Ol܂̎̓G[ɂ }
	else if FResCount < FDatList.Count
		then FThread.ResState := rsNew
		else FThread.ResState := rsNone;
	MaCherieMainForm.RefreshThread(FThread);
	OnTabActivate(Self);

	if FShrinkedLog then
	begin
		{ Ol܂̎̓CXbhŃCxg𔭐 }
		FViewTimer.OnTimer := OnOccurAbornTimer;
		FViewTimer.Interval := 10;
		FViewTimer.Enabled := True;
	end;
end;

procedure TBrowserTabSheet.OnTabActivate(Sender: TObject);
begin
	if (PageControl <> nil) and (FFrame <> nil) then
	begin
		if (PageControl.ActivePageIndex = PageIndex) and (FThread <> nil) then
		begin
			FFrame.BoardText := FThread.BoardName;
			FFrame.TitleText := Format('%s (%d)[%dKB]', [FThread.ThreadTitle, FThread.ResCount, FThread.DatSize div 1024]);
		end;
	end;
end;

procedure TBrowserTabSheet.OnViewTimer(Sender: TObject);
var
	ALine: string;
begin
	FBrowserLock.Enter;
	try
		FViewTimer.Enabled := False;

		if FDatCount > 0 then
		begin
			ALine := FDatList.ToHTML(FDatIndex, FDatCount);
			if ALine <> '' then
				WriteString(ALine);
			Inc(FDatIndex, FDatCount);
			FDatCount := 0;
		end;
	finally
		FBrowserLock.Leave;
	end;
end;

procedure TBrowserTabSheet.OnOccurAbornTimer(Sender: TObject);
begin
	{ Ol܂莞̏ }

	{ ^C}[̒~ }
	FViewTimer.Enabled := False;

	ShowMessage('Ol܂肪܂B');
end;

procedure TBrowserTabSheet.OpenThread(Thread: T2chThreadIniFile);
var
	URL, DatPath, HTML: string;
begin
	{ ^C}[̒~ }
	FViewTimer.Enabled := False;

	{ Xbh̒~ }
	Assert(not FLoadThread.Terminated);
	if FLoadThread.Terminated then Exit;
	FLoadThread.Disconnect := True;
	while not FLoadThread.Suspended do
		Application.ProcessMessages;

	with Thread do
	begin
		URL := Format('http://%s/%s/dat/%s.dat',
			[ServerName, BoardPath, FormatCurr('000000000', ThreadNum)]);
		Caption := ThreadTitle;
	end;
	FThread := Thread;
	Thread.Linked := True;

	FConfig.Enter;
	try
		DatPath := FConfig.GetBaseDir + 'logx\' + ChangeFileExt(FThread.LogPath, '.dat');
	finally
		FConfig.Leave;
	end;

	FBrowserLock.Enter;
	try
		FFirstTime := True;
		FShrinkedLog := False;	{ Ol܂tONA }
		FViewTimer.Interval := 500;
		FViewTimer.OnTimer := OnViewTimer;

		FDatCount := 0;
		FDatIndex := 0;
		FResCount := FThread.ResCount;

		FDatList.Clear;
		FDatMem.Clear;
		
		Browser.AssignDocument;
		OnTabActivate(Self);

		if FileExists(DatPath) then
			FDatList.LoadFromFile(DatPath);

		FDatCount := 25;
		while FDatIndex < FDatList.Count do
		begin
			if (FDatList.Count - FDatIndex) < FDatCount then
				FDatCount := FDatList.Count - FDatIndex;

			HTML := FDatList.ToHTML(FDatIndex, FDatCount);
			WriteString(HTML);

			Inc(FDatIndex, FDatCount);
		end;
		
		FDatCount := 0;

		if Thread.Freezed then
			Exit;
		
		FLoadThread.RangeStart := 0;
		if FThread.DatSize > 0 then
			FLoadThread.RangeStart := FThread.DatSize - 1;
		FLoadThread.Config := FConfig;
		FLoadThread.URL := URL;
		FLoadThread.Resume;
	finally
		FBrowserLock.Leave;
	end;
end;

procedure TBrowserTabSheet.ThreadCatchException(Message: string);
begin
	ShowMessage(Message);
end;

procedure TBrowserTabSheet.WriteString(const S: string);
begin
	OleVariant(Browser.Document as IHTMLDocument2).write(S);
end;

{ TBrowserLoadThread }

procedure TBrowserLoadThread.CatchException;
begin
	FOwner.ThreadCatchException(FExceptionMessage);
end;

constructor TBrowserLoadThread.Create(AOwner: TBrowserTabSheet);
begin
	inherited Create(True);
	FOwner := AOwner;
	FHTTP := TIdAsyncHTTP.Create(nil);
end;

procedure TBrowserLoadThread.DatOnLine(ASender: TObject; const ALineText,
  ATerminator: string; Param: Pointer);
begin
	FLine := ALineText;
	FTerminator := ATerminator;
	Synchronize(OnLine);
	if FDisconnect then
		FHTTP.ForceDisconnect;
end;

destructor TBrowserLoadThread.Destroy;
begin
	FHTTP.Free;
  inherited;
end;

procedure TBrowserLoadThread.EndDatLoad;
begin
	FOwner.EndThreadLoad;
end;

procedure TBrowserLoadThread.Execute;
begin
	while True do
	begin
		if Terminated then Exit;

		FDisconnect := False;
		FFreezed := False;

		FConfig.Enter;
		try
			with FHTTP do
			begin
				Request.Clear;
				ProxyParams.Clear;
				Request.UserAgent := FConfig.CommParams.UserAgent;
				if FConfig.CommParams.ProxyParams.ProxyServer <> '' then
					ProxyParams.Assign(FConfig.CommParams.ProxyParams);
				ReadTimeout := FConfig.CommParams.ReadTimeOut;
			end;
		finally
			FConfig.Leave;
		end;

		if FRangeStart > 0 then
			FHTTP.Request.ContentRangeStart := FRangeStart;
			
		try
			FHTTP.Response.Clear;
			FHTTP.Get(FURL, DatOnLine);
		except
			on E: EIdAsyncHTTPProtocolException do
				if E.ReplyErrorCode = 302 then	{ Found }
					FFreezed := True	{ qɗtOTrueɂ }
				else begin
					FExceptionMessage := E.ClassName + ': ' + E.Message;
					Synchronize(CatchException);
				end;
			on E: Exception do
			begin
				FExceptionMessage := E.ClassName + ': ' + E.Message;
				Synchronize(CatchException);
			end;
		end;

 		Synchronize(EndDatLoad);

		if FDisconnect then
			FDisconnect := False;

		if Terminated then Exit;

		Suspend;
	end;
end;

procedure TBrowserLoadThread.OnLine;
begin
	FOwner.OnDatLine(FLine, FTerminator);
end;

end.

